/**
 * Created by littwin on 28.03.15.
 */
angular.module('frontapp')
    .factory('KeyboardInteger', ['$uibModal', function ($uibModal) {
        /* @ngdoc method
         * @name app.services.ConfirmBox#ConfirmBox
         * @methodOf app.services.ConfirmBox
         *
         * @description Shows a confirm box with yes and no as answers. Returns a promise
         * that gets resolved when user hits yes and gets rejected otherwise.
         * @param {String} Message to confirm.
         * @returns {promise} Promise about result.
         */
        return function (head, headIcon, min, max,unit, value) {
            var dia = $uibModal.open({
                templateUrl: 'views/dialogs/integer.html?lang=' + lang,
                controller: ['$scope', function ($scope) {
                    $scope.headlineIcon = headIcon;
                    $scope.headline = head;
                    $scope.range = ""+min+" "+unit+" - "+max+" "+unit;
                    $scope.unit = unit;
                    $scope.value = ""+value;
                    $scope.valid = true;

                    var validate = function() {
                        if($scope.value == "")
                            $scope.valid = false;
                        else {
                            var v = parseInt($scope.value);
                            $scope.valid = v >= min && v <= max;
                        }
                    };

                    $scope.addChar = function(c) {
                        if($scope.value.length > 6) return;
                        while($scope.value.length > 0 && $scope.value.substr(0,1) == "0")
                            $scope.value = $scope.value.substr(1);
                        $scope.value += c;
                        validate();
                    };

                    $scope.delChar = function() {
                        if($scope.value.length == 0) return;
                        $scope.value = $scope.value.substr(0, $scope.value.length - 1);
                        if($scope.value.length == 0)
                            $scope.value = "0";
                        validate();
                    };

                    $scope.addValue = function(x) {
                        var val = parseInt($scope.value);
                        val += x;
                        if(val >= min && val <= max)
                            $scope.value = ""+val;
                        validate();
                    };

                    $scope.sendValue = function() {
                        if($scope.valid)
                            $scope.$close(parseInt($scope.value));
                    };
                }],
                size: 'sm'
            });
            return dia.result;
        }
    }])
    .factory('KeyboardFloat', ['$uibModal', function ($uibModal) {
        /* @ngdoc method
         * @name app.services.ConfirmBox#ConfirmBox
         * @methodOf app.services.ConfirmBox
         *
         * @description Shows a confirm box with yes and no as answers. Returns a promise
         * that gets resolved when user hits yes and gets rejected otherwise.
         * @param {String} Message to confirm.
         * @returns {promise} Promise about result.
         */
        return function (head, headIcon, min, max,unit, value) {
            var dia = $uibModal.open({
                templateUrl: 'views/dialogs/float.html?lang=' + lang,
                controller: ['$scope', function ($scope) {
                    $scope.headlineIcon = headIcon;
                    $scope.headline = head;
                    $scope.range = ""+min+" "+unit+" - "+max+" "+unit;
                    $scope.unit = unit;
                    $scope.value = ""+value;
                    $scope.valid = true;

                    var validate = function() {
                        if($scope.value == "")
                            $scope.valid = false;
                        else {
                            var v = parseFloat($scope.value);
                            $scope.valid = v >= min && v <= max;
                        }
                    };

                    $scope.addChar = function(c) {
                        if($scope.value.length > 6) return;
                        if(c=='.' && $scope.value.indexOf('.') >= 0) return;
                        while($scope.value.length > 0 && $scope.value.substr(0,1) == "0")
                            $scope.value = $scope.value.substr(1);
                        $scope.value += c;
                        validate();
                    };

                    $scope.delChar = function() {
                        if($scope.value.length == 0) return;
                        $scope.value = $scope.value.substr(0, $scope.value.length - 1);
                        if($scope.value.length == 0)
                            $scope.value = "0";
                        validate();
                    };

                    $scope.addValue = function(x) {
                        var val = parseInt($scope.value);
                        val += x;
                        if(val >= min && val <= max)
                            $scope.value = ""+val;
                        validate();
                    };

                    $scope.sendValue = function() {
                        if($scope.valid)
                            $scope.$close(parseFloat($scope.value));
                    };
                }],
                size: 'sm'
            });
            return dia.result;
        }
    }])
    .factory('FEQuestion', ['$uibModal', function ($uibModal) {
        /* @ngdoc method
         * @name app.services.ConfirmBox#ConfirmBox
         * @methodOf app.services.ConfirmBox
         *
         * @description Shows a confirm box with yes and no as answers. Returns a promise
         * that gets resolved when user hits yes and gets rejected otherwise.
         * @param {String} Message to confirm.
         * @returns {promise} Promise about result.
         */
        return function (head, question, yes, no, danger) {
            var dia = $uibModal.open({
                templateUrl: 'views/dialogs/question.html?lang=' + lang,
                controller: ['$scope', function ($scope) {
                    $scope.infoText = question;
                    $scope.infoHeader = head;
                    $scope.infoYes = yes || "<?php _('Yes') ?>";
                    $scope.infoNo = no || "<?php _('No') ?>";
                    if (!danger) {
                        $scope.icon="fa-question-circle";
                        $scope.infoYesClass = "btn-primary";
                        $scope.infoNoClass = "btn-default";
                    } else {
                        $scope.icon="fa-warning";
                        $scope.infoYesClass = "btn-danger";
                        $scope.infoNoClass = "btn-primary";
                    }
                    $scope.confirmNo = function () {
                        $scope.$dismiss();
                    };
                    $scope.confirmYes = function () {
                        $scope.$close();
                    };

                }],
                size: 'lg'
            });
            return dia.result;
        }
    }])
    .directive('feswitch', ['$timeout', function ($timeout) {
        return {
            restrict: 'E',
            scope: {changed: '&'},
            template: '<div class="btn-group btn-group-justified speed-temperature" ng-click="changeVal()">'+
                '<label class="btn btn-default off-bright btn-md" ng-class="{\'off-bright\':!val,\'on-dark\':val}"><span ng-hide="val"><i class="fa fa-circle-o"></i></span></label>'+
                '<label class="btn btn-default off-dark btn-md" ng-class="{\'off-dark\':!val,\'on-bright\':val}"><span ng-show="val"><i class="fa fa-rotate-90 fa-minus"></i></span></label></div>',
            replace: true,
            require: 'ngModel',
            link: function (scope, elem, attr, controller) {
                controller.$formatters.push(function (newValue) { // listens on ngModel changes
                    if (typeof(newValue) !== "undefined") {
                        $timeout(function () {
                            if (newValue === "false") newValue = false;
                            if (newValue === "true") newValue = true;
                            scope.val = newValue;
                        });
                    }
                });
                scope.changeVal = function() {
                    scope.val = !scope.val;
                    controller.$setViewValue(scope.val);
                    if (scope.changed) {
                        scope.changed({value: scope.val});
                    }
                };
                $timeout(function () {
                    scope.val = controller.$modelValue || false;
                });
            }
        };
    }])
;